@@ -1,6 +1,6 @@
 <?php
 /**
-* @version V4.54 5 Nov 2004 (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
+* @version V4.94 23 Jan 2007 (c) 2000-2007 John Lim (jlim#natsoft.com.my). All rights reserved.
 * Released under both BSD license and Lesser GPL library license.
 * Whenever there is any discrepancy between the two licenses,
 * the BSD license will take precedence.
@@ -22,14 +22,19 @@ class ADODB_informix extends ADODB_informix72 {
 	var $databaseType = "informix";
 	var $hasTop = 'FIRST';
 	var $ansiOuter = true;
+	
+	function IfNull( $field, $ifNull )
+	{
+		return " NVL($field, $ifNull) "; // if Informix 9.X or 10.X
+	}
 }
 
 class ADORecordset_informix extends ADORecordset_informix72 {
 	var $databaseType = "informix";
-
+	
 	function ADORecordset_informix($id,$mode=false)
 	{
 		$this->ADORecordset_informix72($id,$mode);
 	}
 }
-?>
+?>
\ No newline at end of file
