@@ -1,15 +1,25 @@
 <?php
-	# Mantis - a php based bugtracking system
-	# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
-	# Copyright (C) 2002 - 2005  Mantis Team   - mantisbt-dev@lists.sourceforge.net
-	# This program is distributed under the terms and conditions of the GPL
-	# See the README and LICENSE files for details
+# Mantis - a php based bugtracking system
+
+# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
+# Copyright (C) 2002 - 2007  Mantis Team   - mantisbt-dev@lists.sourceforge.net
+
+# Mantis is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or
+# (at your option) any later version.
+#
+# Mantis is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Mantis.  If not, see <http://www.gnu.org/licenses/>.
 
 	# --------------------------------------------------------
-	# $Id: columns_api.php,v 1.12 2005/08/10 19:59:13 thraxisp Exp $
+	# $Id: columns_api.php,v 1.21.2.1 2007-10-13 22:35:18 giallu Exp $
 	# --------------------------------------------------------
-?>
-<?php
 
 	# --------------------
 	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
@@ -53,6 +63,19 @@
 		}
 	}
 
+	# --------------------
+	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
+	function print_column_title_duplicate_id( $p_sort, $p_dir, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
+		if ( $p_columns_target != COLUMNS_TARGET_CSV_PAGE ) {
+			echo '<td>';
+			print_view_bug_sort_link( lang_get( 'duplicate_id' ), 'duplicate_id', $p_sort, $p_dir, $p_columns_target );
+			print_sort_icon( $p_dir, $p_sort, 'duplicate_id' );
+			echo '</td>';
+		} else {
+			echo lang_get( 'duplicate_id' );
+		}
+	}
+
 	# --------------------
 	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
 	function print_column_title_reporter_id( $p_sort, $p_dir, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
@@ -84,7 +107,7 @@
 	function print_column_title_priority( $p_sort, $p_dir, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
 		if ( $p_columns_target != COLUMNS_TARGET_CSV_PAGE ) {
 			echo '<td>';
-			print_view_bug_sort_link( 'P', 'priority', $p_sort, $p_dir, $p_columns_target );
+			print_view_bug_sort_link( lang_get( 'priority_abbreviation' ), 'priority', $p_sort, $p_dir, $p_columns_target );
 			print_sort_icon( $p_dir, $p_sort, 'priority' );
 			echo '</td>';
 		} else {
@@ -157,6 +180,19 @@
 		}
 	}
 
+	# --------------------
+	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
+	function print_column_title_target_version( $p_sort, $p_dir, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
+		if ( $p_columns_target != COLUMNS_TARGET_CSV_PAGE ) {
+			echo '<td>';
+			print_view_bug_sort_link( lang_get( 'target_version' ), 'target_version', $p_sort, $p_dir, $p_columns_target );
+			print_sort_icon( $p_dir, $p_sort, 'target_version' );
+			echo '</td>';
+		} else {
+			echo lang_get( 'target_version' );
+		}
+	}
+
 	# --------------------
 	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
 	function print_column_title_view_state( $p_sort, $p_dir, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
@@ -510,11 +546,27 @@
 		echo '</td>';
 	}
 
+	# --------------------
+	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
+	function print_column_eta( $p_row, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
+		if ( $p_columns_target != COLUMNS_TARGET_CSV_PAGE ) {
+			echo '<td class="center">', get_enum_element( 'eta', $p_row['eta'] ), '</td>';
+		} else {
+			echo get_enum_element( 'eta', $p_row['eta'] );
+		}
+	}
+
+	# --------------------
+	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
+	function print_column_resolution( $p_row, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
+		echo '<td class="center">', get_enum_element( 'resolution', $p_row['resolution'] ), '</td>';
+	}
+
 	# --------------------
 	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
 	function print_column_status( $p_row, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
 		echo '<td class="center">';
-		printf( '<u><a title="%s">%s</a></u>'
+		printf( '<span class="issue-status" title="%s">%s</span>'
 			, get_enum_element( 'resolution', $p_row['resolution'] )
 			, get_enum_element( 'status', $p_row['status'] )
 		);
@@ -562,12 +614,24 @@
 		echo '</td>';
 	}
 
+	# --------------------
+	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
+	function print_column_date_submitted( $p_row, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
+		$t_date_submitted = date( config_get( 'short_date_format' ), $p_row['date_submitted'] );
+
+		echo '<td class="center">', $t_date_submitted, '</td>';
+	}
+
 	# --------------------
 	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
 	function print_column_summary( $p_row, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
 		global $t_icon_path;
 
-		$t_summary = string_attribute( $p_row['summary'] );
+		if ( $p_columns_target == COLUMNS_TARGET_CSV_PAGE ) {
+			$t_summary = string_attribute( $p_row['summary'] );
+		} else {
+			$t_summary = string_display_line_links( $p_row['summary'] );
+		}
 
 		echo '<td class="left">', $t_summary;
 		if ( VS_PRIVATE == $p_row['view_state'] ) {
@@ -579,4 +643,14 @@
 		}
 		echo '</td>';
 	}
-?>
\ No newline at end of file
+	
+	# --------------------
+	# $p_columns_target: see COLUMNS_TARGET_* in constant_inc.php
+	function print_column_target_version( $p_row, $p_columns_target = COLUMNS_TARGET_VIEW_PAGE ) {
+		echo '<td>';
+		if ( access_has_bug_level( config_get( 'roadmap_view_threshold' ), $p_row['id'] ) ) {
+			echo $p_row['target_version'];
+		}
+		echo '</td>';
+	}
+?>
