@@ -1,12 +1,24 @@
 <?php
-	# Mantis - a php based bugtracking system
-	# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
-	# Copyright (C) 2002 - 2004  Mantis Team   - mantisbt-dev@lists.sourceforge.net
-	# This program is distributed under the terms and conditions of the GPL
-	# See the README and LICENSE files for details
+# Mantis - a php based bugtracking system
+
+# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
+# Copyright (C) 2002 - 2007  Mantis Team   - mantisbt-dev@lists.sourceforge.net
+
+# Mantis is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or
+# (at your option) any later version.
+#
+# Mantis is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Mantis.  If not, see <http://www.gnu.org/licenses/>.
 
 	# --------------------------------------------------------
-	# $Id: current_user_api.php,v 1.28.14.1 2006/09/23 06:48:45 vboctor Exp $
+	# $Id: current_user_api.php,v 1.31.2.1 2007-10-13 22:35:21 giallu Exp $
 	# --------------------------------------------------------
 
 	$t_core_dir = dirname( __FILE__ ).DIRECTORY_SEPARATOR;
@@ -113,7 +125,10 @@
 
 		if ( !is_blank( $f_filter_string ) ) {
 			if( is_numeric( $f_filter_string ) ) {
-				$t_filter = unserialize( token_get_value( $f_filter_string ) );
+				$t_token = token_get_value( TOKEN_FILTER );
+				if ( null != $t_token ) {
+					$t_filter = unserialize( $t_token );
+				}
 			} else {
 				$t_filter = unserialize( $f_filter_string );
 			}
@@ -121,7 +136,7 @@
 			return false;
 		} else {
 			$t_user_id = auth_get_current_user_id();
-			return user_get_bug_filter( $t_user_id, $p_project_id );
+			$t_filter = user_get_bug_filter( $t_user_id, $p_project_id );
 		}
 
 		$t_filter = filter_ensure_valid_filter( $t_filter );
