@@ -1,12 +1,24 @@
 <?php
-	# Mantis - a php based bugtracking system
-	# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
-	# Copyright (C) 2002 - 2004  Mantis Team   - mantisbt-dev@lists.sourceforge.net
-	# This program is distributed under the terms and conditions of the GPL
-	# See the README and LICENSE files for details
+# Mantis - a php based bugtracking system
+
+# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
+# Copyright (C) 2002 - 2007  Mantis Team   - mantisbt-dev@lists.sourceforge.net
+
+# Mantis is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or
+# (at your option) any later version.
+#
+# Mantis is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Mantis.  If not, see <http://www.gnu.org/licenses/>.
 
 	# --------------------------------------------------------
-	# $Id: graph_api.php,v 1.31.12.1 2006/04/18 14:22:05 thraxisp Exp $
+	# $Id: graph_api.php,v 1.36.2.1 2007-10-13 22:35:29 giallu Exp $
 	# --------------------------------------------------------
 
 	if ( ON == config_get( 'use_jpgraph' ) ) {
@@ -26,7 +38,6 @@
 			'arial' => FF_ARIAL,
 			'verdana' => FF_VERDANA,
 			'courier' => FF_COURIER,
-			'book' => FF_BOOK,
 			'comic' => FF_COMIC,
 			'times' => FF_TIMES,
 			'georgia' => FF_GEORGIA,
@@ -55,7 +66,9 @@
 
 		$graph = new Graph( $p_graph_width, $p_graph_height );
 		$graph->img->SetMargin(40,40,40,170);
-		$graph->img->SetAntiAliasing();
+		if ( ON == config_get_global( 'jpgraph_antialias' ) ) {
+			$graph->img->SetAntiAliasing();
+		}
 		$graph->SetScale('textlin');
 		$graph->SetMarginColor('white');
 		$graph->SetFrame(false);
@@ -78,7 +91,7 @@
 		$p1->SetFillColor('yellow');
 		$p1->SetWidth(0.8);
 		$graph->Add($p1);
-		if ( ON == config_get( 'show_queries_count' ) ) {
+		if ( helper_show_queries() ) {
 			$graph->subtitle->Set( db_count_queries() . ' queries (' . db_count_unique_queries() . ' unique) (' . db_time_queries() . 'sec)' );
 			$graph->subtitle->SetFont( $t_graph_font, FS_NORMAL, 8 );
 		}
@@ -112,7 +125,9 @@
 		#defines margin according to height
 		$graph = new Graph( $p_graph_width, $p_graph_height );
 		$graph->img->SetMargin( 45, 35, 35, $p_baseline );
-		$graph->img->SetAntiAliasing();
+		if ( ON == config_get_global( 'jpgraph_antialias' ) ) {
+			$graph->img->SetAntiAliasing();
+		}
 		$graph->SetScale('textlin');
 		$graph->SetMarginColor('white');
 		$graph->SetFrame(false);
@@ -157,7 +172,7 @@
 		$gbplot = new GroupBarPlot(array($p1,$p3,$p2));
 		$graph->Add($gbplot);
 
-		if ( ON == config_get( 'show_queries_count' ) ) {
+		if ( helper_show_queries() ) {
 			$graph->subtitle->Set( db_count_queries() . ' queries (' . db_count_unique_queries() . ' unique) (' . db_time_queries() . 'sec)' );
 			$graph->subtitle->SetFont( $t_graph_font, FS_NORMAL, 8 );
 		}
@@ -177,7 +192,9 @@
 
 		$graph = new Graph(250,400);
 		$graph->img->SetMargin(35,35,35,150);
-		$graph->img->SetAntiAliasing();
+		if ( ON == config_get_global( 'jpgraph_antialias' ) ) {
+			$graph->img->SetAntiAliasing();
+		}
 		$graph->SetScale('textlin');
 		$graph->SetMarginColor('white');
 		$graph->SetFrame(false);
@@ -205,7 +222,7 @@
         $gbplot = new GroupBarPlot(array($p1,$p2,$p3));
 
         $graph->Add($gbplot);
-		if ( ON == config_get( 'show_queries_count' ) ) {
+		if ( helper_show_queries() ) {
 			$graph->subtitle->Set( db_count_queries() . ' queries (' . db_count_unique_queries() . ' unique)' );
 		}
 		$graph->Stroke();
@@ -231,7 +248,7 @@
 		$graph->legend->Pos($p_poshorizontal, $p_posvertical);
 		$graph->legend->SetFont( $t_graph_font );
 
-		$p1 = new PiePlot3d( array_values( $p_metrics ) );
+		$p1 = new PiePlot3d( array_values( $p_metrics ) ); // should be reversed?
 		$p1->SetTheme('earth');
 		#$p1->SetTheme("sand");
 		$p1->SetCenter($p_center);
@@ -244,7 +261,7 @@
 		$p1->value->SetFont( $t_graph_font );
 
 		$graph->Add($p1);
-		if ( ON == config_get( 'show_queries_count' ) ) {
+		if ( helper_show_queries() ) {
 			$graph->subtitle->Set( db_count_queries() . ' queries (' . db_count_unique_queries() . ' unique) (' . db_time_queries() . 'sec)' );
 			$graph->subtitle->SetFont( $t_graph_font, FS_NORMAL, 8 );
 		}
@@ -268,7 +285,9 @@
 
 		$graph = new Graph( $p_graph_width, $p_graph_height );
 		$graph->img->SetMargin(40,40,40,170);
-		$graph->img->SetAntiAliasing();
+		if ( ON == config_get_global( 'jpgraph_antialias' ) ) {
+			$graph->img->SetAntiAliasing();
+		}
 		$graph->SetScale('linlin');
 		$graph->SetMarginColor('white');
 		$graph->SetFrame(false);
@@ -310,13 +329,69 @@
 		$p2->SetLegend( lang_get( 'legend_resolved' ) );
 		$graph->Add($p2);
 
-		if ( ON == config_get( 'show_queries_count' ) ) {
+		if ( helper_show_queries() ) {
 			$graph->subtitle->Set( db_count_queries() . ' queries (' . db_count_unique_queries() . ' unique) (' . db_time_queries() . 'sec)' );
 			$graph->subtitle->SetFont( $t_graph_font, FS_NORMAL, 8 );
 		}
 		$graph->Stroke();
 	}
 
+
+	# --------------------
+	function graph_bydate( $p_metrics, $p_labels, $p_title, $p_graph_width = 300, $p_graph_height = 380 ){
+
+		$t_graph_font = graph_get_font();
+		error_check( is_array( $p_metrics ) ? count($p_metrics) : 0, lang_get( 'by_date' ) );
+
+		$graph = new Graph( $p_graph_width, $p_graph_height );
+		$graph->img->SetMargin(40,140,40,100);
+		if ( ON == config_get_global( 'jpgraph_antialias' ) ) {
+			$graph->img->SetAntiAliasing();
+		}
+		$graph->SetScale('linlin');
+		$graph->SetMarginColor('white');
+		$graph->SetFrame(false);
+		$graph->title->Set( $p_title . ' ' . lang_get( 'by_date' ) );
+		$graph->title->SetFont( $t_graph_font, FS_BOLD );
+
+		$graph->legend->Pos(0.01,0.05,'right','top');
+		$graph->legend->SetShadow(false);
+		$graph->legend->SetFillColor('white');
+		$graph->legend->SetLayout(LEGEND_VERT);
+		$graph->legend->SetFont( $t_graph_font );
+
+		$graph->yaxis->scale->ticks->SetDirection(-1);
+		$graph->yaxis->SetFont( $t_graph_font );
+        $graph->yaxis->scale->SetAutoMin(0); 
+        
+		if ( FF_FONT2 <= $t_graph_font ) {
+			$graph->xaxis->SetLabelAngle(60);
+		} else {
+			$graph->xaxis->SetLabelAngle(90);	# can't rotate non truetype fonts
+		}
+		$graph->xaxis->SetLabelFormatCallback('graph_date_format');
+		$graph->xaxis->SetFont( $t_graph_font );
+
+        $t_line_colours = config_get( 'graph_colors' );
+        $t_count_colours = count($t_line_colours);
+        $t_lines = count($p_metrics) - 1;
+        $t_line = array();
+        for ( $i = 1; $i <= $t_lines; $i++) {
+		    $t_line[$i] = new LinePlot($p_metrics[$i], $p_metrics[0]);
+		    $t_line[$i]->SetColor($t_line_colours[$i % $t_count_colours]);
+		    $t_line[$i]->SetCenter();
+	        $t_line[$i]->SetLegend( lang_get_defaulted( $p_labels[$i] ) );
+		    $graph->Add($t_line[$i]);
+        }
+
+		if ( helper_show_queries() ) {
+			$graph->subtitle->Set( db_count_queries() . ' queries (' . db_count_unique_queries() . ' unique) (' . db_time_queries() . 'sec)' );
+			$graph->subtitle->SetFont( $t_graph_font, FS_NORMAL, 8 );
+		}
+		$graph->Stroke();
+	}
+	
+	
 	# --------------------
 	# utilities
 	# --------------------
