@@ -1,24 +1,35 @@
 <?php
-	# Mantis - a php based bugtracking system
-	# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
-	# Copyright (C) 2002 - 2004  Mantis Team   - mantisbt-dev@lists.sourceforge.net
-	# This program is distributed under the terms and conditions of the GPL
-	# See the README and LICENSE files for details
+# Mantis - a php based bugtracking system
+
+# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
+# Copyright (C) 2002 - 2007  Mantis Team   - mantisbt-dev@lists.sourceforge.net
+
+# Mantis is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or
+# (at your option) any later version.
+#
+# Mantis is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Mantis.  If not, see <http://www.gnu.org/licenses/>.
 
 	# --------------------------------------------------------
-	# $Id: news_delete.php,v 1.23 2005/05/16 19:20:32 marcelloscata Exp $
+	# $Id: news_delete.php,v 1.23.18.1 2007-10-13 22:34:02 giallu Exp $
 	# --------------------------------------------------------
-?>
-<?php
+
 	require_once( 'core.php' );
 
 	$t_core_path = config_get( 'core_path' );
 
 	require_once( $t_core_path.'news_api.php' );
-?>
-<?php
-	$f_news_id = gpc_get_int( 'news_id' );
 
+	form_security_validate( 'news_delete' );
+
+	$f_news_id = gpc_get_int( 'news_id' );
 	$row = news_get_row( $f_news_id );
 
 	# This check is to allow deleting of news items that were left orphan due to bug #3723
@@ -31,6 +42,8 @@
 
 	news_delete( $f_news_id );
 
+	form_security_purge( 'news_delete' );
+
 	$t_redirect_url = 'news_menu_page.php';
 	html_page_top1();
 	html_meta_redirect( $t_redirect_url );
