@@ -1,12 +1,24 @@
 <?php
-	# Mantis - a php based bugtracking system
-	# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
-	# Copyright (C) 2002 - 2004  Mantis Team   - mantisbt-dev@lists.sourceforge.net
-	# This program is distributed under the terms and conditions of the GPL
-	# See the README and LICENSE files for details
+# Mantis - a php based bugtracking system
+
+# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
+# Copyright (C) 2002 - 2007  Mantis Team   - mantisbt-dev@lists.sourceforge.net
+
+# Mantis is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or
+# (at your option) any later version.
+#
+# Mantis is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Mantis.  If not, see <http://www.gnu.org/licenses/>.
 
 	# --------------------------------------------------------
-	# $Id: view_all_set.php,v 1.57.4.1.2.1.4.1 2006/04/06 00:28:52 thraxisp Exp $
+	# $Id: view_all_set.php,v 1.67.2.1 2007-10-13 22:34:48 giallu Exp $
 	# --------------------------------------------------------
 ?>
 <?php require_once( 'core.php' ) ?>
@@ -43,6 +55,30 @@
 		$f_show_category = array( $f_show_category );
 	}
 
+	$f_platform = array();
+	if ( is_array( gpc_get( 'platform', null ) ) ) {
+		$f_platform = gpc_get_string_array( 'platform', META_FILTER_ANY );
+	} else {
+		$f_platform = gpc_get_string( 'platform', META_FILTER_ANY );
+		$f_platform = array( $f_platform );
+	}
+
+	$f_os = array();
+	if ( is_array( gpc_get( 'os', null ) ) ) {
+		$f_os = gpc_get_string_array( 'os', META_FILTER_ANY );
+	} else {
+		$f_os = gpc_get_string( 'os', META_FILTER_ANY );
+		$f_os = array( $f_os );
+	}
+
+	$f_os_build = array();
+	if ( is_array( gpc_get( 'os_build', null ) ) ) {
+		$f_os_build = gpc_get_string_array( 'os_build', META_FILTER_ANY );
+	} else {
+		$f_os_build = gpc_get_string( 'os_build', META_FILTER_ANY );
+		$f_os_build = array( $f_os_build );
+	}
+
 	$f_show_severity = array();
 	if ( is_array( gpc_get( 'show_severity', null ) ) ) {
 		$f_show_severity = gpc_get_string_array( 'show_severity', META_FILTER_ANY );
@@ -83,6 +119,14 @@
 		$f_handler_id = array( $f_handler_id );
 	}
 
+	$f_project_id = array();
+	if ( is_array( gpc_get( 'project_id', null ) ) ) {
+		$f_project_id = gpc_get_int_array( 'project_id', META_FILTER_CURRENT );
+	} else {
+		$f_project_id = gpc_get_int( 'project_id', META_FILTER_CURRENT );
+		$f_project_id = array( $f_project_id );
+	}
+
 	$f_show_resolution = array();
 	if ( is_array( gpc_get( 'show_resolution', null ) ) ) {
 		$f_show_resolution = gpc_get_string_array( 'show_resolution', META_FILTER_ANY );
@@ -115,6 +159,14 @@
 		$f_fixed_in_version = array( $f_fixed_in_version );
 	}
 	
+	$f_target_version = array();
+	if ( is_array( gpc_get( 'target_version', null ) ) ) {
+		$f_target_version = gpc_get_string_array( 'target_version', META_FILTER_ANY );
+	} else {
+		$f_target_version = gpc_get_string( 'target_version', META_FILTER_ANY );
+		$f_target_version = array( $f_target_version );
+	}
+
 	$f_show_profile = array();
 	if ( is_array( gpc_get( 'show_profile', null ) ) ) {
 		$f_show_profile = gpc_get_string_array( 'show_profile', META_FILTER_ANY );
@@ -161,7 +213,10 @@
 	$f_search				= gpc_get_string( 'search', '' );
 	$f_and_not_assigned		= gpc_get_bool( 'and_not_assigned' );
 	$f_do_filter_by_date	= gpc_get_bool( 'do_filter_by_date' );
-	$f_view_state			= gpc_get_string( 'view_state', META_FILTER_ANY );
+	$f_view_state			= gpc_get_int( 'view_state', META_FILTER_ANY );
+
+	$f_tag_string			= gpc_get_string( 'tag_string', '' );
+	$f_tag_select			= gpc_get_int( 'tag_select', '0' );
 
 	$t_custom_fields 		= custom_field_get_ids(); # @@@ (thraxisp) This should really be the linked ids, but we don't know the project
 	$f_custom_fields_data 	= array();
@@ -255,7 +310,7 @@
 	$t_hide_status_default = config_get( 'hide_status_default' );
 
 	# show bugs per page
-	if ( $f_per_page < 1 ) {
+	if ( $f_per_page < 0 ) {
 		$f_per_page = config_get( 'default_limit_view' );
 	}
 
@@ -329,11 +384,13 @@
 	switch ( $f_type ) {
 		# New cookie
 		case '0':
+				log_event( LOG_FILTERING, 'view_all_set.php: New cookie' );
 				$t_setting_arr = array();
 
 				break;
 		# Update filters
 		case '1':
+				log_event( LOG_FILTERING, 'view_all_set.php: Update filters' );
 				$t_setting_arr['_version'] = $t_cookie_version;
 				$t_setting_arr['_view_type'] = $f_view_type;
 				$t_setting_arr['show_category'] = $f_show_category;
@@ -343,6 +400,7 @@
 				$t_setting_arr['highlight_changed'] = $f_highlight_changed;
 				$t_setting_arr['reporter_id'] = $f_reporter_id;
 				$t_setting_arr['handler_id'] = $f_handler_id;
+				$t_setting_arr['project_id'] = $f_project_id;
 				$t_setting_arr['sort'] = $f_sort;
 				$t_setting_arr['dir'] = $f_dir;
 				$t_setting_arr['start_month'] = $f_start_month;
@@ -359,6 +417,7 @@
 				$t_setting_arr['show_version'] = $f_show_version;
 				$t_setting_arr['do_filter_by_date'] = $f_do_filter_by_date;
 				$t_setting_arr['fixed_in_version'] = $f_fixed_in_version;
+				$t_setting_arr['target_version'] = $f_target_version;
 				$t_setting_arr['show_priority'] = $f_show_priority;
 				$t_setting_arr['user_monitor'] = $f_user_monitor;
 				$t_setting_arr['view_state'] = $f_view_state;
@@ -367,10 +426,16 @@
 				$t_setting_arr['relationship_type'] = $f_relationship_type;
 				$t_setting_arr['relationship_bug'] = $f_relationship_bug;
 				$t_setting_arr['show_profile'] = $f_show_profile;
-
+				$t_setting_arr['platform'] = $f_platform;
+				$t_setting_arr['os'] = $f_os;
+				$t_setting_arr['os_build'] = $f_os_build;
+				$t_setting_arr['tag_string'] = $f_tag_string;
+				$t_setting_arr['tag_select'] = $f_tag_select;
 				break;
 		# Set the sort order and direction
 		case '2':
+				log_event( LOG_FILTERING, 'view_all_set.php: Set the sort order and direction.' );
+
 				# We only need to set those fields that we are overriding
 				$t_setting_arr['sort'] = $f_sort;
 				$t_setting_arr['dir'] = $f_dir;
@@ -379,6 +444,8 @@
 		# This is when we want to copy another query from the
 		# database over the top of our current one
 		case '3':
+				log_event( LOG_FILTERING, 'view_all_set.php: Copy another query from database' );
+
 				$t_filter_string = filter_db_get_filter( $f_source_query_id );
 				# If we can use the query that we've requested,
 				# grab it. We will overwrite the current one at the
@@ -394,6 +461,8 @@
 				break;
 		# Generalise the filter
 		case '4':
+				log_event( LOG_FILTERING, 'view_all_set.php: Generalise the filter' );
+
 				$t_setting_arr['show_category']	= array( META_FILTER_ANY );
 				$t_setting_arr['reporter_id'] 	= array( META_FILTER_ANY );
 				$t_setting_arr['handler_id'] 	= array( META_FILTER_ANY );
@@ -401,6 +470,7 @@
 				$t_setting_arr['show_version'] 	= array( META_FILTER_ANY );
 				$t_setting_arr['show_priority']	= array( META_FILTER_ANY );
 				$t_setting_arr['fixed_in_version']	= array( META_FILTER_ANY );
+				$t_setting_arr['target_version']	= array( META_FILTER_ANY );
 				$t_setting_arr['user_monitor'] 		= array( META_FILTER_ANY );
 				$t_setting_arr['relationship_type'] = -1;
 				$t_setting_arr['relationship_bug'] = 0;
@@ -417,16 +487,19 @@
 				break;
 		# Just set the search string value
 		case '5':
+				log_event( LOG_FILTERING, 'view_all_set.php: Search Text' );
 				$t_setting_arr['search'] = $f_search;
 
 				break;
 		# Just set the view_state (simple / advanced) value
 		case '6':
+				log_event( LOG_FILTERING, 'view_all_set.php: View state (simple/advanced)' );
 				$t_setting_arr['_view_type'] = $f_view_type;
 
 				break;
 		# does nothing. catch all case
 		default:
+				log_event( LOG_FILTERING, 'view_all_set.php: default - do nothing' );
 				break;
 	}
 
@@ -454,7 +527,7 @@
 	}
 
 	if ( $f_temp_filter ) {
-		$t_token_id = token_add( $t_settings_serialized, TOKEN_FILTER);
+		$t_token_id = token_set( TOKEN_FILTER, $t_settings_serialized );
 		$t_redirect_url = $t_redirect_url . '?filter=' . $t_token_id;
 		html_meta_redirect( $t_redirect_url, 0 );
 	} else {
